local class = require("silo.lib.class")

local Protocol = class("silo.Protocol")

function Protocol:new(ctx)
  self.bus = ctx.bus
  self.messaging = ctx:require("silo.messaging")
  self.taskQueue = ctx:require("silo.task_queue")
end

function Protocol:announce(peerType)
  self.messaging:listen(function(sender, message)
    if message.type == "announce" then
      self.bus:fire("peer.found", sender, message)

      if message.resp == nil then
        self.messaging:sendDirect(sender, {
          type = "announce",
          announce = peerType,
          resp = message.seq
        })
      end
    end
  end)

  self.messaging:send({ type = "announce", announce = peerType })
end

function Protocol:listen(messageType, handler)
  self.messaging:listen(function(sender, message)
    if message.type ~= messageType then return end

    self.taskQueue:enqueue(function()
      local response, callback, broadcast = handler(sender, message)

      if response ~= nil then
        response.resp = message.seq

        if broadcast then
          self.messaging:send(response, callback)
        else
          self.messaging:sendDirect(sender, response, callback)
        end
      end
    end)
  end)
end

return Protocol
